require 'vispan/szenario.rb'
require 'vispan/relation.rb'
require 'vispan/node.rb'
require 'vispan/output_graphic.rb'
require 'fileutils'

module Vispan
  attr_reader :szenarios

  class WebPreview
    def call(env)
      @szenarios = []
      separate_szenarios(@input)

      @szenarios.each(&:assign_ranks)

      visualize(@output)
      file_ending = File.extname(@output).delete('.')
      [200, { 'Content-Type' => "image/#{file_ending}" }, [IO.read(@output)]]
    end

    def initialize(files)
      @input = files[:input]
      @output = files[:output]
    end

    def separate_szenarios(input_file)
      @single_szenario = Szenario.new
      @relation_lines = []
      @lines = File.readlines(input_file)
      @lines.each_with_index do |line, index|
        if line[0] == '-'
          create_new_szenario(line)
        elsif (line == "\n" && !@relation_lines.empty?) || (index == @lines.length - 1)
          create_new_relation(line)
        elsif line != "\n"
          @relation_lines << line
        end
      end
      @szenarios << @single_szenario
    end

    def create_new_szenario(line)
      if @single_szenario.title == ''
        @single_szenario.set_title(line)
      else
        @szenarios << @single_szenario
        @single_szenario = Szenario.new(line)
      end
    end

    def create_new_relation(line)
      @relation_lines << line if line != "\n"
      relation = Relation.new(@relation_lines, @szenarios.length)
      @single_szenario.add_relation(relation)
      @relation_lines = []
    end

    def visualize(output_file)
      graphic = OutputGraphic.new(@szenarios, output_file)
      graphic.generate
    end
  end
end
