require 'vispan/config.rb'

class GraphWriter
  def build_string(szenarios)
    "Digraph G{ #{get_general_settings}#{szenarios.map.with_index {|szenario, index| build_szenario(szenario, index)}.join('')}}"
  end

  def build_szenario(szenario, index)
    node_definitions = szenario.nodes.map {|node| write_node(node)}.join('')
    relation_definitions = szenario.relations.map {|relation| write_relations(relation, szenario)}.join('')
    effect_definitions = szenario.relations.map {|relation| write_effects(relation)}.join('')

    "\n\nsubgraph cluster#{index.to_s} {label=\"#{szenario.title}\"#{find_attributes(szenario)};\n#{node_definitions}\n#{relation_definitions}#{effect_definitions}}"
  end

  def write_node(node)
    "#{node.name} [label=\"#{node.label}\" #{find_attributes(node)}]\n"
  end

  def write_relations(relation, szenario)
    output_port = 'e'
    input_port = 'w'
    sorting_order = ''

    if relation.get_direction < 0
      previous_node_name = szenario.get_previous_helper_node(relation)
      output_port = 'w'
      input_port = 'e'
      sorting_order = "{rank = same; #{previous_node_name}; #{relation.helper.name};}"
    end

    "#{relation.start.name}:#{output_port} -> #{relation.helper.name}:#{input_port} [dir=\"none\" #{find_attributes(relation)}];
     #{relation.helper.name}:#{output_port} -> #{relation.stop.name}:#{input_port} [dir=\"forward\" #{find_attributes(relation)}];
     #{sorting_order}"
  end

  def write_effects(relation)
    "subgraph cluster#{relation.start.name} {label = \"Side effects\"; labeljust = \"c\"; style=\"rounded\"; {rank = same; #{relation.helper.name}; #{relation.effects.map(&:name).join(';')};};}" unless relation.effects.empty?
  end

  def find_attributes(element)
    type = element.class.to_s.upcase
    GraphWriter.const_get(type)[element.type.to_sym].map {|attribute| "; #{attribute[0]}=\"#{attribute[1]}\""}.join('')
  end

  def get_general_settings
    GraphWriter.const_get('GENERAL_SETTINGS').map {|attribute| "#{attribute[0]} [#{attribute[1].map {|attri| "#{attri[0]}=\"#{attri[1]}\""}.join(';')}]"}.join(';')
  end
end
