require 'vispan/graphwriter.rb'

class Szenario
  attr_accessor :title, :relations, :nodes, :type

  def initialize(line = '')
    @title = line.gsub(/--/, '').strip
    @relations = []
    @nodes = []
    @type = 'szenario'
  end

  def build_szenario(index)
    graph = GraphWriter.new
    node_definitions = @nodes.map {|node| graph.write_node(node)}.join('')
    relation_definitions = @relations.map {|relation| graph.write_relations(relation)}.join('')
    effect_definitions = @relations.map {|relation| graph.write_effects(relation)}.join('')

    "\n\nsubgraph cluster#{index.to_s} {bgcolor=aliceblue; label=\"#{title}\";\n#{node_definitions}\n#{relation_definitions}#{effect_definitions}}"
  end

  def set_title(line)
    @title = line.gsub(/--/, '').strip
  end

  def add_relation(relation)
    add_nodes([relation.start, relation.stop, relation.helper, relation.effects])
    replace_with_existing_nodes(relation, @nodes)
    @relations << relation
  end

  def add_nodes(nodes_array)
    nodes_array = nodes_array.flatten
    nodes_array.map {|node| @nodes << node if is_not_saved(node, @nodes)}
  end

  def is_not_saved(node, szenario_nodes)
    szenario_nodes.each do |szenario_node|
      return false if szenario_node.name == node.name
    end
  end

  def replace_with_existing_nodes(relation, nodes)
    nodes.each do |node|
      relation.start = node if relation.start.name == node.name
      relation.stop = node if relation.stop.name == node.name
    end
  end

  def assign_ranks
    rank = @relations.map {|relation|
      if relation.start.rank == 0
        relation.start.rank = 1
        relation.stop.rank = 2 unless relation.stop.rank > 0
      elsif relation.stop.rank == 0
        relation.stop.rank = relation.start.rank + 1
      end
    }
  end

  def get_previous_helper_node(relation2)
    @relations.each do |relation|
      if relation2.start.name == relation.stop.name
        return relation.helper.name
      end
    end
  end
end
